import { DynamicModule, Module, Provider } from '@nestjs/common';
import { PubSubService } from './pub-sub.service';
import {
  createOptionsProvider,
  createPubSubProvider,
} from './pub-sub.providers';
import { PubSubOptions } from './interfaces/pub-sub-options.interface';
import { PubSubAsyncConfig } from './interfaces/pub-sub-async-config.interface';
import { getPubSubToken } from './utils/token';
import { PUB_SUB_CONFIG_TOKEN } from './pub-sub.constants';

@Module({
  providers: [PubSubService],
})
export class PubSubModule {
  public static forRoot(options: PubSubOptions): DynamicModule {
    const providers = [createPubSubProvider(options)];
    return {
      global: true,
      module: PubSubModule,
      providers,
      exports: providers,
    };
  }
  public static forRootAsync(config: PubSubAsyncConfig) {
    const providers: Provider[] = [
      createOptionsProvider(config),
      {
        provide: getPubSubToken(config.name),
        inject: [PUB_SUB_CONFIG_TOKEN],
        useFactory: (options: PubSubOptions) => {
          return createPubSubProvider({
            name: config.name,
            ...options,
          });
        },
      },
    ];
    return {
      global: true,
      module: PubSubModule,
      imports: config.imports,
      providers,
      exports: providers,
    };
  }
}
